#ifndef SOLVER_HPP
#define SOLVER_HPP

// copy of the solution, to be used in generators

#include <iostream>
#include <vector>
#include <algorithm>

using std::vector;
using std::pair;
using std::swap;
using std::make_pair;

template<typename Compare>
class DSU {
  vector<int> root;
  Compare cmp;

public:
  DSU () {
  }
  
  DSU (int n, Compare _cmp) {
    root = vector<int> (n);
    for (int i = 0; i < n; i++) {
      root[i] = i;
    }

    cmp = _cmp;
  }

  int find (int u) {
    if (root[u] != u) {
      root[u] = find(root[u]);
    }
    return root[u];
  }

  void merge (int u, int v) {
    u = find(u);
    v = find(v);
    if (cmp(v, u)) {
      swap(u, v);
    }

    root[v] = u;
  }
};

class LevelComparator {
  vector<int> priority;

public:
  LevelComparator() {
  }
  
  LevelComparator(vector<int> _priority) : priority(_priority) {
  }

  bool operator() (int p, int q) {
    return priority[p] < priority[q];
  }
};

// Data structure that maintains the state of a graph's bridges - being able
// to add edges and dynamically find the bridges of the graph.
// We do so by maintaining an union-find data structure that keeps vertices
// u and v in the same class iff u can be reached from v without traversing
// a bridge. We'll call the classes of the DSU as "bridge-separated components".
//
// The DS can be thought of as a graph whose vertices are bridge-separated components
// and whose edges are the bridges of the original graph. It turns out that
// the result will be a tree; we'll root that tree at an arbitrary vertex.
// If bridge-separated components u and v are joined by an edge, we will
// merge u, v and all components between u, v and their LCA, into a single
// component. Notice that the amortized complexity of this (over all queries)
// will be no more than O(n log n), because we can only make n - 1 merge operations
// before the tree only has one vertex.
//
// Implementation-wise, we build the bridge tree on a spanning tree of the original
// graph, this spanning tree is injected to the constructor. Every bridge-separated
// component will be a subgraph of this spanning tree; the only vertices whose parent
// is outside its bridge-separated component are those whose parents are the topmost
// vertices of each bridge-separated components. The DSU is set up in a way that
// these vertices will also be the representatives of the bridge-separated components.
class BridgeTree {
  DSU<LevelComparator> dsu;
  vector<int> parent;
  vector<int> lvl;
  int bridge_count;

public:
  BridgeTree () {
  }

  BridgeTree (vector<int> _parent, vector<int> _lvl)
    : dsu(_parent.size(), LevelComparator(_lvl)), parent(_parent),
      lvl(_lvl), bridge_count((int) _parent.size() - 1) {
  }

  void add_edge (int u, int v) {
    u = dsu.find(u);
    v = dsu.find(v);

    while (u != v) {
      if (lvl[u] < lvl[v]) {
        swap(u, v);
      }

      dsu.merge(u, parent[u]);
      u = dsu.find(u);
      bridge_count--;
    }
  }

  int get_bridge_count () {
    return bridge_count;
  }
};

// Answers the actual queries. We make use of BridgeTree to count the number
// of bridges on the fly. Notice that we can't initialize BridgeTree until
// the graph is connected, so we keep our own adjacency list until the first
// query. Once we have to answer the query, the problem statement guarantees
// that the graph is connected. At that point we can initialize the BridgeTree
// and direct the edge-adding to BridgeTree directly.
class Solver {
  int n;
  vector<vector<int>> adj;
  BridgeTree bridge_tree;
  bool is_initialized;

  void build (int u, int p, vector<int> &parent, vector<int> &lvl,
              vector<pair<int, int>> &extra_edges) {
    parent[u] = p;
    lvl[u] = lvl[p] + 1;

    for (int nxt : adj[u]) {
      if (lvl[nxt] == 0) { 
        build(nxt, u, parent, lvl, extra_edges);
      } else if (lvl[nxt] > lvl[u]) {
        extra_edges.push_back(make_pair(nxt, u));
      }
    }
  }
  
  void ensure_initialized () {
    if (is_initialized) {
      return;
    }

    vector<int> parent (n);
    vector<int> lvl (n);
    vector<pair<int, int>> extra_edges;
    build(0, 0, parent, lvl, extra_edges);
    bridge_tree = BridgeTree(parent, lvl);
    for (auto e : extra_edges) {
      bridge_tree.add_edge(e.first, e.second);
    }
    is_initialized = true;
  }

public:
  Solver (int _n) : n(_n), adj(_n, vector<int> (0)), is_initialized(false) {
  }

  void add_edge (int u, int v) {
    if (is_initialized) {
      bridge_tree.add_edge(u, v);
    } else {
      adj[u].push_back(v);
      adj[v].push_back(u);
    }
  }

  // Is it possible to paint the edges s.t that k edges are cheap, the
  // rest are expensive, and the MST is unique?
  bool can_price_uniquely (int k) {
    ensure_initialized();
    
    // Cheap edges must not form cycles, therefore there can not be more than
    // n - 1 cheap edges.
    if (k >= n) {
      return false;
    }
    
    // Otherwise, within each bridge-separated component of size S we must have
    // exactly S - 1 cheap edges, forming a spanning tree of that component.
    // if there were less, we would have several cheap-connected components
    // within the component. Since the bridge-separated component does not
    // contain bridges, there would be several equally-costly options to
    // connect the cheap-connected components with expensive edges.
    //
    // In other words, the number of cheap and expensive edges used inside
    // bridge-separated components are fixed. We can only vary how many
    // bridges are expensive and how many are cheap.
    int bridgec = bridge_tree.get_bridge_count();
    int must_be_cheap = n - 1 - bridgec;
    return k >= must_be_cheap;
  }

  int get_min_cheap_edges () {
    ensure_initialized();
    return n - 1 - bridge_tree.get_bridge_count();
  }
};

#endif
